@@ -973,7 +973,9 @@ copy_object (ibfd, obfd)
 
   /* Copy architecture of input file to output file.  */
   if (!bfd_set_arch_mach (obfd, bfd_get_arch (ibfd),
-			  bfd_get_mach (ibfd)))
+			  bfd_get_mach (ibfd))
+      && (ibfd->target_defaulted
+	  || bfd_get_arch (ibfd) != bfd_get_arch (obfd)))
     non_fatal (_("Warning: Output file cannot represent architecture %s"),
 	       bfd_printable_arch_mach (bfd_get_arch (ibfd),
 					bfd_get_mach (ibfd)));
@@ -1684,13 +1686,15 @@ copy_section (ibfd, isection, obfdarg)
   sec_ptr osection;
   bfd_size_type size;
   long relsize;
+  flagword flags;
 
   /* If we have already failed earlier on,
      do not keep on generating complaints now.  */
   if (status != 0)
     return;
 
-  if ((bfd_get_section_flags (ibfd, isection) & SEC_DEBUGGING) != 0
+  flags = bfd_get_section_flags (ibfd, isection);
+  if ((flags & SEC_DEBUGGING) != 0
       && (strip_symbols == STRIP_DEBUG
 	  || strip_symbols == STRIP_UNNEEDED
 	  || strip_symbols == STRIP_ALL
@@ -1698,6 +1702,9 @@ copy_section (ibfd, isection, obfdarg)
 	  || convert_debugging))
     return;
 
+  if ((flags & SEC_GROUP) != 0)
+    return;
+
   p = find_section_list (bfd_section_name (ibfd, isection), false);
 
   if (sections_removed && p != NULL && p->remove)
