@@ -1,21 +1,32 @@
 /*
- * Copyright (c) 1983, 1998, 2001, 2002 Regents of the University of California.
- * All rights reserved.
+ * Copyright (c) 1983, 1993, 1998, 2001, 2002
+ *      The Regents of the University of California.  All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted
- * provided that: (1) source distributions retain this entire copyright
- * notice and comment, and (2) distributions including binaries display
- * the following acknowledgement:  ``This product includes software
- * developed by the University of California, Berkeley and its contributors''
- * in the documentation or other materials provided with the distribution
- * and in all advertising materials mentioning features or use of this
- * software. Neither the name of the University nor the names of its
- * contributors may be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the University nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
+
 #include "libiberty.h"
 #include "gprof.h"
 #include "search_list.h"
@@ -85,6 +96,7 @@ static char *default_excluded_list[] =
 
 #define OPTION_DEMANGLE		(150)
 #define OPTION_NO_DEMANGLE	(OPTION_DEMANGLE + 1)
+#define OPTION_DEMANGLER	(OPTION_NO_DEMANGLE + 1)
 
 static struct option long_options[] =
 {
@@ -112,6 +124,7 @@ static struct option long_options[] =
   {"all-lines", no_argument, 0, 'x'},
   {"demangle", optional_argument, 0, OPTION_DEMANGLE},
   {"no-demangle", no_argument, 0, OPTION_NO_DEMANGLE},
+  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"directory-path", required_argument, 0, 'I'},
   {"display-unused-functions", no_argument, 0, 'z'},
   {"min-count", required_argument, 0, 'm'},
@@ -161,9 +174,10 @@ Usage: %s [-[abcDhilLsTvwxyz]] [-[ACeEfFJnNOpPqQZ][name]] [-I dirs]\n\
 	[--no-static] [--print-path] [--separate-files]\n\
 	[--static-call-graph] [--sum] [--table-length=len] [--traditional]\n\
 	[--version] [--width=n] [--ignore-non-functions]\n\
-	[--demangle[=STYLE]] [--no-demangle]\n\
+	[--demangle[=%s]]\n\
+	[--no-demangle] [--demangler=dso:function]\n\
 	[image-file] [profile-file...]\n"),
-	   whoami);
+	   whoami, get_demangler_list ());
   if (status == 0)
     fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
   done (status);
@@ -178,6 +192,7 @@ main (argc, argv)
   char **sp, *str;
   Sym **cg = 0;
   int ch, user_specified = 0;
+  const char *demangler = NULL, *style = NULL;
 
 #if defined (HAVE_SETLOCALE) && defined (HAVE_LC_MESSAGES)
   setlocale (LC_MESSAGES, "");
@@ -439,29 +454,21 @@ This program is free software.  This program has absolutely no warranty.\n"));
 	case OPTION_DEMANGLE:
 	  demangle = true;
 	  if (optarg != NULL)
-	    {
-	      enum demangling_styles style;
-
-	      style = cplus_demangle_name_to_style (optarg);
-	      if (style == unknown_demangling)
-		{
-		  fprintf (stderr,
-			   _("%s: unknown demangling style `%s'\n"),
-			   whoami, optarg);
-		  xexit (1);
-		}
-
-	      cplus_demangle_set_style (style);
-	   }
+	    style = optarg;
 	  break;
 	case OPTION_NO_DEMANGLE:
 	  demangle = false;
 	  break;
+	case OPTION_DEMANGLER:
+	  demangler = optarg;
+	  break;
 	default:
 	  usage (stderr, 1);
 	}
     }
 
+  init_demangler (style, NULL, demangler);
+
   /* Don't allow both ordering options, they modify the arc data in-place.  */
   if ((user_specified & STYLE_FUNCTION_ORDER)
       && (user_specified & STYLE_FILE_ORDER))
