@@ -148,6 +148,8 @@ typedef struct {
      fpor overlaps.  */
   boolean check_section_addresses;
 
+  /* It is used to specify where to find some multilib files. */
+  char *multilib_dir;
 } args_type;
 
 extern args_type command_line;
